//JOHN PALMISANO		11-26-03

//LAST MODIFIED			11-27-03

//modified fwr03.h (awesomified it) for 4 wheel drive T2

//only works for early 2003 cerebellums, so go away



#include <16F877.h>

#include <877reg.h>

#include "adc.h"



#use rs232(baud=115200, parity=N, bits=8, xmit=PIN_C6, rcv=PIN_C7)



// for LEDs

#define YELLOW 1

#define GREEN 0



// for buttons

#define A 0

#define B 1



// for wheels

#define LEFT 0

#define RIGHT 1



// ADC ports

#define A0 0

#define A1 1

#define A2 2

#define A3 3

#define A5 4

#define E0 5

#define E1 6

#define E2 7



// SERVO control vars/functions - not for the faint of heart!

// Specify Servo Port

#define SERVOPORT 3

#define SERVO_PORT PORTD

#define SERVO_TRIS TRISD



char servo_state = 0;

char servo_count = 0;

char servo_curr = 0;

//char servo_mask = 0;

char servo_switch = 1;

char servo_pos[8] = {0,0,0,0,0,0,0,0}; // servo_pos[7] is reserved for a boolean check if heat is found

char servo_free = 0;

//#bit servo_free = _servo_free.0



#INT_RTCC

// servo control variables

void servo_isr(void) {



	servo_free = 0;

	if (servo_state == 1) {

		// next intr in 0.6 ms

		TMR0 = 180;



		if (servo_pos[servo_curr] != 0)

			SERVO_PORT = SERVO_PORT | servo_switch; // output hi



		servo_switch = servo_switch << 1;

		if (servo_switch == 0) servo_switch = 1;



		servo_state = servo_state + 1;

		servo_count = 0;

		servo_free = 1;

	}

	else if (servo_state == 2) {

		TMR0 = 255 - servo_pos[servo_curr];

		servo_state = 3;

	}

	else if (servo_state == 3) {

		SERVO_PORT = 0; // output lo

		TMR0 = servo_pos[servo_curr];

		servo_curr = servo_curr + 1;

		servo_curr = servo_curr & 0x07;

		servo_state = 1;

	}

	T0IF = 0;

}



inline void servo_init(void) {

    OPTION_REG = 132; // bit 7 = 1 for portB pull-up and bit2 is for prescaler

	// TMR0 @ 1:32, prescaler to timer0 (for servo control)

    TRISD = 0; // port D as outputs	for servo commands!

    T0IE = 1; //T0IE = 1 ie enable TMR0 overflow bit - T0IE is bit 5

    delay_ms(1);

}



// pause program for x milliseconds.

void pause(unsigned long int time)

{

  unsigned int timemod;



  while(time > 0)

  {

    if(time > 255)

    {

      timemod = 255;

      time -= 255;

    }

    else

    {

      timemod = time;

      time = 0;

    }



    delay_ms(timemod);

  }

}



// turn a specified(0 or 1) LED on.

inline void LEDon(int which)

{

  if(which == 0)

  {

    output_high(PIN_B4);

  }

  else

  {

    output_high(PIN_B5);

  }

}



// turn a specified(0 or 1) LED off.

inline void LEDoff(int which)

{

  if(which == 0)

  {

    output_low(PIN_B4);

  }

  else

  {

    output_low(PIN_B5);

  }

}



// takes -100 to 100 speed, converts to servo 255 to 0

void motors(signed int velLeft, signed int velRight)

{

  int pulseLeft;

  int pulseRight;



  // converts -100 to 100 to the full range of servo motion

  // centers at 127, the 0 spot of a servo.

  pulseLeft = 127 - (int)((3*((signed long int)velLeft))/5);

  pulseRight = 127 + (int)((3*((signed long int)velRight))/5);



  // sets the given servo to the given speed.

  servo_pos[4]=pulseLeft+1;//modified

  servo_pos[5]=pulseRight+1;

  servo_pos[6]=pulseLeft+1;

  servo_pos[7]=pulseRight+2;

}



// takes a -90 to 90 angle, converts to servo 255 to 0

void steer(signed int TL, signed int TR, signed int BL, signed int BR)

{

  int pulseTL;

  int pulseTR;

  int pulseBL;

  int pulseBR;

  

  // converts -100 to 100 to the full range of servo motion

  // centers at 127, the 0 spot of a servo.

  pulseTL = 127 + (int)((4*((signed long int)TL))/3);

  pulseTR = 127 + (int)((4*((signed long int)TR))/3);

  pulseBL = 127 + (int)((4*((signed long int)BL))/3);

  pulseBR = 127 + (int)((4*((signed long int)BR))/3);



  // sets the given servo to the given speed.

  servo_pos[0]=pulseTL+4;//modified because servo misaligned

  servo_pos[1]=pulseTR;

  servo_pos[2]=pulseBL-2;//modified because servo misaligned

  servo_pos[3]=pulseBR-6;//modified because servo misaligned

}



//look im spinning, im spinning!

inline void spin()

{

  int center=62;

  steer(-center,center,center,-center);

  delay_ms(100);

}



//aligns servos

inline void straighten()

{

  int center=0;

  steer(center,center,center,center);

  delay_ms(100);

}



//crab like sideways motion thingy

inline void crab(int angle)

{

  steer(angle,angle,angle,angle);

}



// read from a given analog port

inline unsigned int analog(int which)

{

	int sense;



	sense = 1*(int)adc_read(which);

	delay_ms(1);



	if(sense < 0)

		sense += 256;



	return sense;

}



inline int button(int which)

{

   int state;

   if(which == A)

	state = input(PIN_B0);

    else

	state = input(PIN_B2);



   return state;

}



inline void waitforbutton(int which)

{

  while(!button(which))

    pause(10);

}



inline void cereb_init()

{

  GIE = 1; // Enable Interrupts (servos only work with enabled interrupts)



  servo_init();  // Initialize servos

  servo_state = 1;  // Enable servos



  adc_init(0);



  LEDoff(GREEN);

  LEDoff(YELLOW);



  delay_ms(1);

}

